##
## Copyright 2014-2019 the original author or authors.
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
## either express or implied. See the License for the specific language
## governing permissions and limitations under the License.
##
${database.allClassCopyright}package ${myBaseEntityPackageName};
#set ($myClassName = "${myBaseObjectClassName}")

import java.io.Serializable;
import java.util.*;

#if ($table.hasSimpleDtoJsonicDecoration())
import net.arnx.jsonic.JSONHint;
#end
#if ($table.hasSimpleDtoJsonPullParserDecoration())
import net.vvakame.util.jsonpullparser.annotation.*;
#end
#if ($table.hasSimpleDtoJacksonDecoration())
import com.fasterxml.jackson.annotation.JsonFormat;
#end
#if ($database.isSimpleDtoClassificationDeployment() && $table.hasClassification())
import ${database.simpleCDefPackage}.${database.simpleCDefClass};
#end
#if ($table.hasRelation())
import ${myExtendedEntityPackageName}.*;
#end

/**
 * The simple DTO of ${table.basicInfoDispString}. <br>
#if ($table.isCommentForJavaDocValid())
 * ${table.commentForJavaDoc}
#end
 * <pre>
 * [primary-key]
 *     ${table.primaryKeyNameCommaString}
 *
 * [column]
 *     ${table.columnNameCommaString}
 *
 * [sequence]
 *     ${table.definedSequenceName}
 *
 * [identity]
 *     ${table.identityColumnName}
 *
 * [version-no]
 *     ${table.versionNoColumnName}
 *
 * [foreign-table]
 *     ${table.foreignTableNameCommaString}
 *
 * [referrer-table]
 *     ${table.referrerTableNameCommaString}
 *
 * [foreign-property]
 *     ${table.foreignPropertyNameCommaString}
 *
 * [referrer-property]
 *     ${table.referrerPropertyNameCommaString}
 * </pre>
 * @author ${database.classAuthor}
 */
#if ($table.hasSimpleDtoJsonPullParserDecoration())
@JsonModel(decamelize = false)
#end
public abstract class $myClassName implements Serializable {

    // ===================================================================================
    //                                                                          Definition
    //                                                                          ==========
    /** The serial version UID for object serialization. (Default) */
    private static final long serialVersionUID = 1L;

    // ===================================================================================
    //                                                                           Attribute
    //                                                                           =========
    // -----------------------------------------------------
    //                                                Column
    //                                                ------
#foreach ($col in $table.columns)
    /** ${col.aliasExpression}${col.name}: {${col.columnDefinitionLineDisp}} */
#if ($col.hasSimpleDtoJsonPullParserDecoration())
    @JsonKey
#end
    protected ${col.javaNative} ${col.simpleDtoVariableName};

#end
    // -----------------------------------------------------
    //                                              Internal
    //                                              --------
    /** The modified properties for this DTO. */
## GWT does not allow to use final mark to migrate the data
#if ($database.isSimpleDtoGwtDecorationSuppressJavaDependency())
    protected Set<String> __modifiedProperties = new LinkedHashSet<String>();
#else
    protected final Set<String> __modifiedProperties = new LinkedHashSet<String>();
#end

    // ===================================================================================
    //                                                                         Constructor
    //                                                                         ===========
    public ${myClassName}() {
    }

    // ===================================================================================
    //                                                                 Modified Properties
    //                                                                 ===================
    public Set<String> mymodifiedProperties() {
        return __modifiedProperties;
    }

    public void clearModifiedInfo() {
        __modifiedProperties.clear();
    }

    public boolean hasModification() {
        return !__modifiedProperties.isEmpty();
    }

#if ($database.isSimpleDtoClassificationDeployment() && $table.hasClassification())
    // ===================================================================================
    //                                                             Classification Property
    //                                                             =======================
#foreach ($col in $table.columns)
#if ($col.hasClassification())
#set ($classificationTop = $database.getClassificationTop($col.classificationName))
    /**
     * Set the value of ${col.uncapitalisedJavaName} as the classification of ${col.classificationName}. <br>
#if ($classificationTop.hasTopComment())
     * ${classificationTop.topCommentForJavaDoc}
#end
     * @param cdef The instance of classification definition (as ENUM type). (NullAllowed)
     */
    public void set${col.javaName}As${col.classificationName}(${database.simpleCDefClass}.${col.classificationName} cdef) {
#if ($col.isJavaNativeNumberObject())
        set${col.javaName}(cdef != null ? ${col.javaNative}.valueOf(cdef.code()) : null);
#elseif ($col.isJavaNativeBooleanObject())
        set${col.javaName}(cdef != null ? ${col.javaNative}.valueOf(cdef.code()) : null);
#else
        set${col.javaName}(cdef != null ? cdef.code() : null);
#end
    }

    /**
     * Get the value of ${col.uncapitalisedJavaName} as the classification of ${col.classificationName}. <br>
#if ($classificationTop.hasTopComment())
     * ${classificationTop.topCommentForJavaDoc}
#end
     * <p>It's treated as case insensitive and if the code value is null, it returns null.</p>
     * @return The instance of classification definition (as ENUM type). (NullAllowed)
     */
    public ${database.simpleCDefClass}.${col.classificationName} get${col.javaName}As${col.classificationName}() {
        return ${database.simpleCDefClass}.${col.classificationName}.codeOf(get${col.javaName}());
    }

#end
#end
    // ===================================================================================
    //                                                              Classification Setting
    //                                                              ======================
#foreach ($col in $table.columns)
#if ($col.hasClassification())
#foreach ($classificationElement in $col.classificationTop.classificationElementList)
    /**
     * Set the value of ${col.uncapitalisedJavaName} as ${classificationElement.name}. <br>
     * $database.buildClassificationApplicationCommentForJavaDoc($classificationElement)
     */
    public void set${col.javaName}_${classificationElement.name}() {
        set${col.javaName}As${col.classificationName}(${database.simpleCDefClass}.${col.classificationName}.${classificationElement.name});
    }

#end
#end
#end
    // ===================================================================================
    //                                                        Classification Determination
    //                                                        ============================
#foreach ($col in $table.columns)
#if ($col.hasClassification())
#foreach ($classificationElement in $col.classificationTop.classificationElementList)
    /**
     * Is the value of ${col.uncapitalisedJavaName} '${classificationElement.name}'? <br>
     * $database.buildClassificationApplicationCommentForJavaDoc($classificationElement)
     * <p>It's treated as case insensitive and if the code value is null, it returns false.</p>
     * @return The determination, true or false.
     */
    public boolean is${col.JavaName}${classificationElement.name}() {
        ${database.simpleCDefClass}.${col.classificationName} cdef = get${col.javaName}As${col.classificationName}();
        return cdef != null ? cdef.equals(${database.simpleCDefClass}.${col.classificationName}.${classificationElement.name}) : false;
    }

#end
#end
#end
#end
    // ===================================================================================
    //                                                                       Foreign Table
    //                                                                       =============
#foreach ($fk in $table.foreignKeys)
#set ($foreignEntityClassName = "${fk.foreignTableExtendedSimpleDtoClassName}")
#set ($foreignVariableName = "${fk.simpleDtoForeignVariableName}")
#if ($col.hasSimpleDtoJsonPullParserDecoration())
    @JsonKey
#end
    protected ${foreignEntityClassName} ${foreignVariableName};

    public ${foreignEntityClassName} get${fk.foreignJavaBeansRulePropertyNameInitCap}() {
        return ${foreignVariableName};
    }

    public void set${fk.foreignJavaBeansRulePropertyNameInitCap}(${foreignEntityClassName} ${fk.foreignPropertyName}) {
        this.${foreignVariableName} = ${fk.foreignPropertyName};
    }

#end
#foreach ($referrer in $table.referrerAsOneList)
#set ($referrerEntityClassName = "${referrer.referrerTableExtendedSimpleDtoClassName}")
#set ($referrerVariableName = "${referrer.simpleDtoReferrerAsOneVariableName}")
#if ($col.hasSimpleDtoJsonPullParserDecoration())
    @JsonKey
#end
    protected ${referrerEntityClassName} ${referrerVariableName};

    public ${referrerEntityClassName} get${referrer.referrerJavaBeansRulePropertyNameAsOneInitCap}() {
        return ${referrerVariableName};
    }

    public void set${referrer.referrerJavaBeansRulePropertyNameAsOneInitCap}(${referrerEntityClassName} ${referrer.referrerJavaBeansRulePropertyNameAsOne}) {
        this.${referrerVariableName} = ${referrer.referrerJavaBeansRulePropertyNameAsOne};
    }

#end
    // ===================================================================================
    //                                                                      Referrer Table
    //                                                                      ==============
#foreach ($referrer in $table.referrerAsManyList)
#set ($referrerEntityClassName = "${referrer.referrerTableExtendedSimpleDtoClassName}")
#set ($referrerVariableName = "${referrer.simpleDtoReferrerVariableName}")
#if ($col.hasSimpleDtoJsonPullParserDecoration())
    @JsonKey
#end
    protected List<${referrerEntityClassName}> $referrerVariableName;

    public List<${referrerEntityClassName}> get${referrer.referrerJavaBeansRulePropertyNameInitCap}() {
#if (!$database.isOneToManyReturnNullIfNonSelect())
        if (${referrerVariableName} == null) { ${referrerVariableName} = new ArrayList<${referrerEntityClassName}>(); }
#end
        return ${referrerVariableName};
    }

    public void set${referrer.referrerJavaBeansRulePropertyNameInitCap}(List<${referrerEntityClassName}> ${referrer.referrerPropertyName}) {
        this.${referrerVariableName} = ${referrer.referrerPropertyName};
    }

#end
    // ===================================================================================
    //                                                                      Basic Override
    //                                                                      ==============
    public boolean equals(Object other) {
        if (other == null || !(other instanceof $myClassName)) { return false; }
        final $myClassName otherEntity = ($myClassName)other;
#foreach ($col in $table.equalsHashcodeColumnList)
        if (!helpComparingValue(get${col.javaName}(), otherEntity.get${col.javaName}())) { return false; }
#end
        return true;
    }

    protected boolean helpComparingValue(Object value1, Object value2) {
        if (value1 == null && value2 == null) { return true; }
        return value1 != null && value2 != null && value1.equals(value2);
    }

    public int hashCode() {
        int result = 17;
        result = xCH(result, "${table.tableDbName}");
#foreach ($col in $table.equalsHashcodeColumnList)
        result = xCH(result, get${col.JavaName}());
#end
        return result;
    }
    protected int xCH(int result, Object value) { // calculateHashcode()
        if (value == null) {
            return result;
        }
        return (31 * result) + (value instanceof byte[] ? ((byte[]) value).length : value.hashCode());
    }

    public int instanceHash() {
        return super.hashCode();
    }

    public String toString() {
        String c = ", ";
        StringBuilder sb = new StringBuilder();
#foreach ($col in $table.columns)
#if ($col.isJavaNativeUtilDate())
        sb.append(c).append(xfUD(get${col.javaName}()));
#elseif ($col.isJavaNativeByteArray())
        sb.append(c).append(xfBA(get${col.javaName}()));
#else
        sb.append(c).append(get${col.javaName}());
#end
#end
        if (sb.length() > 0) { sb.delete(0, c.length()); }
        sb.insert(0, "{").append("}");
        return sb.toString();
    }
#if ($table.hasUtilDateColumn())
#if ($database.isSimpleDtoGwtDecorationSuppressJavaDependency())
    protected String xfUD(java.util.Date date) { // formatUtilDate()
        return date != null ? date.toString() : null;
    }
#else
    protected String xfUD(java.util.Date date) { // formatUtilDate()
        return date != null ? new java.text.SimpleDateFormat(xgDP()).format(date) : null;
    }
    protected String xgDP() { // getDatePattern
#if ($database.isDatabaseOracle())
        return "yyyy-MM-dd HH:mm:ss"; // time parts for Oracle only
#else
        return "yyyy-MM-dd";
#end
    }
#end
#end
#if ($table.hasByteArrayColumn())
    protected String xfBA(byte[] bytes) { // formatByteArray()
        return "byte[" + (bytes != null ? String.valueOf(bytes.length) : "null") + "]";
    }
#end

    // ===================================================================================
    //                                                                            Accessor
    //                                                                            ========
#foreach ($col in $table.columns)
    /**
     * [get] ${col.aliasExpression}${col.name}: {${col.columnDefinitionLineDisp}} <br>
#if ($col.isCommentForJavaDocValid())
     * ${col.commentForJavaDoc}
#end
     * @return The value of the column '${col.name}'. (NullAllowed)
     */
#if ($col.hasSimpleDtoJsonicDecorationDatePattern())
    @JSONHint(format="${col.simpleDtoJsonicDecorationDatePattern}")
#end
#if ($col.hasSimpleDtoJsonicDecorationTimestampPattern())
    @JSONHint(format="${col.simpleDtoJsonicDecorationTimestampPattern}")
#end
#if ($col.hasSimpleDtoJsonicDecorationTimePattern())
    @JSONHint(format="${col.simpleDtoJsonicDecorationTimePattern}")
#end
#if ($col.hasSimpleDtoJacksonDecorationDatePattern())
    @JsonFormat(pattern="${col.simpleDtoJacksonDecorationDatePattern}")
#end
#if ($col.hasSimpleDtoJacksonDecorationTimestampPattern())
    @JsonFormat(pattern="${col.simpleDtoJacksonDecorationTimestampPattern}")
#end
#if ($col.hasSimpleDtoJacksonDecorationTimePattern())
    @JsonFormat(pattern="${col.simpleDtoJacksonDecorationTimePattern}")
#end
    public ${col.javaNative} get${col.javaName}() {
        return ${col.simpleDtoVariableName};
    }

    /**
     * [set] ${col.aliasExpression}${col.name}: {${col.columnDefinitionLineDisp}} <br>
#if ($col.isCommentForJavaDocValid())
     * ${col.commentForJavaDoc}
#end
     * @param ${col.uncapitalisedJavaName} The value of the column '${col.name}'. (NullAllowed)
     */
    public void set${col.javaName}(${col.javaNative} ${col.uncapitalisedJavaName}) {
        __modifiedProperties.add("${col.javaBeansRulePropertyName}");
        this.${col.simpleDtoVariableName} = ${col.uncapitalisedJavaName};
    }

#end
}
